class
	P_MUSIC_STATIC

inherit
	CE_UTILITIES
	S_STDLIB_STATIC
	S_SDL_SDL_AUDIO_STATIC
	S_CTYPE_STATIC
	S_STRING_STATIC
	S_SDL_SDL_ERROR_STATIC
	S_SDL_SDL_RWOPS_STATIC
	S_SDL_SDL_TIMER_STATIC

feature {ANY} -- public attributes

	used_mixer: CE_STATIC [CE_ARRAY [SD_SDL_AUDIOSPEC]]
		once ("PROCESS")
			Result.item := create {CE_CLASS_ARRAY [SD_SDL_AUDIOSPEC]}.make (<<1>>)
		end

	music_active: CE_STATIC [INTEGER_32]
		once ("PROCESS")
			Result.item := 1
		end

	music_stopped: CE_STATIC [INTEGER_32]
		once ("PROCESS")
			Result.item := 0
		end

	music_loops: CE_STATIC [INTEGER_32]
		once ("PROCESS")
			Result.item := 0
		end

	music_cmd: CE_STATIC [CE_POINTER [INTEGER_8]]
		once ("PROCESS")
			Result.item := create {CE_INTEGER_8_POINTER}.make_shared (default_pointer)
		end

	music_playing: CE_STATIC [CE_POINTER [PD_MIX_MUSIC]]
		once ("PROCESS")
			Result.item := create {CE_CLASS_POINTER [PD_MIX_MUSIC]}.make_shared (default_pointer)
		end

	music_volume: CE_STATIC [INTEGER_32]
		once ("PROCESS")
			Result.item := 128
		end

	ms_per_step: CE_STATIC [INTEGER_32]
		once ("PROCESS")
		end

	music_decoders: CE_STATIC [CE_POINTER [CE_POINTER [INTEGER_8]]]
		once ("PROCESS")
			Result.item := create {CE_POINTER_POINTER [INTEGER_8]}.make_shared (default_pointer)
		end

	num_decoders_0: CE_STATIC [INTEGER_32]
		once ("PROCESS")
			Result.item := 0
		end

	music_finished_hook: CE_STATIC [CE_ROUTINE]
		once ("PROCESS")
			Result.item := create {CE_ROUTINE}.make_shared (default_pointer)
		end

feature {ANY} -- public routines

	mix_get_num_music_decoders: INTEGER_32
		do
			Result := num_decoders_0.item
		end

	mix_get_music_decoder (a_index: INTEGER_32): CE_POINTER [INTEGER_8]
		local
			l_return: BOOLEAN
		do
			if a_index < 0 then
				Result := (create {CE_INTEGER_8_POINTER}.make_shared (default_pointer))
				l_return := True
			elseif a_index >= num_decoders_0.item then
				Result := (create {CE_INTEGER_8_POINTER}.make_shared (default_pointer))
				l_return := True
			end
			if not l_return then
				Result := (music_decoders.item[a_index])
			end
		end

	add_music_decoder (a_decoder: CE_POINTER [INTEGER_8])
		local
			l_ptr: CE_POINTER [ANY]
			l_tmp_0: INTEGER_32
			l_return: BOOLEAN
		do
			l_ptr := (realloc (create {CE_VOID_POINTER}.make_cast (music_decoders.item), (num_decoders_0.item + 1).to_natural_32 * {PLATFORM}.Pointer_bytes.to_natural_32))
			if l_ptr.is_void then
				l_return := True
			end
			if not l_return then
				music_decoders.item := create {CE_POINTER_POINTER [INTEGER_8]}.make_cast (l_ptr)
				l_tmp_0 := num_decoders_0.item
				num_decoders_0.item := num_decoders_0.item + 1
				music_decoders.item[l_tmp_0] := a_decoder
			end
		end

	mix_hook_music_finished (a_music_finished: CE_ROUTINE)
		do
			sdl_lock_audio
			music_finished_hook.item := a_music_finished
			sdl_unlock_audio
		end

	music_halt_or_loop: INTEGER_32
		local
			l_current_fade: NATURAL_32
			l_return: BOOLEAN
		do
			if not (music_internal_playing).to_boolean then
				if music_loops.item.to_boolean then
					music_loops.item := music_loops.item - 1
					l_current_fade := music_playing.item.item.fading
					music_internal_play (music_playing.item, 0.0).do_nothing
					music_playing.item.item.fading := l_current_fade
				else
					music_internal_halt
					if (not music_finished_hook.item.is_void) then
						music_finished_hook.item.call ([])
					end
					Result := 0
					l_return := True
				end
			end
			if not l_return then
				Result := 1
			end
		end

	music_mixer (a_udata: CE_POINTER [ANY]; a_stream: CE_POINTER [NATURAL_8]; a_len: INTEGER_32)
		local
			l_left: INTEGER_32
			l_volume: INTEGER_32
			l_fade_step: INTEGER_32
			l_fade_steps: INTEGER_32
			l_tmp: INTEGER_32
			l_return: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_left := 0
					if music_playing.item.is_void then
						l_pos := -1 + 7 -- skip if block
					end
				when 1 then -- if:
						if not (music_active.item.to_boolean) then
							l_pos := -1 + 7 -- skip if block
						end
				when 2 then -- if:
						if music_playing.item.item.fading /= {NATURAL_32} 0 then
							l_tmp := music_playing.item.item.fade_step
							music_playing.item.item.fade_step := (music_playing.item.item.fade_step) + 1
							if l_tmp < music_playing.item.item.fade_steps then
								l_fade_step := music_playing.item.item.fade_step
								l_fade_steps := music_playing.item.item.fade_steps
								if music_playing.item.item.fading = {NATURAL_32} 1 then
									l_volume := (music_volume.item * (l_fade_steps - l_fade_step)) // l_fade_steps
								else
									l_volume := (music_volume.item * l_fade_step) // l_fade_steps
								end
								music_internal_volume (l_volume)
							else
								if music_playing.item.item.fading = {NATURAL_32} 1 then
									music_internal_halt
									if (not music_finished_hook.item.is_void) then
										music_finished_hook.item.call ([])
									end
									l_return := True
								end
								if not l_return then
									music_playing.item.item.fading := {PC_MIX_FADING}.mix_no_fading
								end
							end
						end
						if not l_return then
							music_halt_or_loop.do_nothing
							if not (music_internal_playing).to_boolean then
								l_return := True
							end
						end
						if l_return then
							l_pos := -1 + 7 -- skip if block
						end
				when 3 then -- if:
						inspect music_playing.item.item.type -- switch 1 head
						when 2 then l_pos := -1 + 4
						when 6 then l_pos := -1 + 5
						else l_pos := -1 + 6
						end
				when 4 then -- switch 1, case 2u
						l_left := (create {P_WAVESTREAM_STATIC}).wavstream_play_some (a_stream, a_len)
						l_pos := -1 + 7 -- switch 1; break
				when 5 then -- switch 1, case 6u
						l_left := a_len - (((create {P_DYNAMIC_MP3_STATIC}).smpeg.item.smpeg_play_audio).integer_32_item ([music_playing.item.item.data.mp3, a_stream, a_len]))
						l_pos := -1 + 7 -- switch 1; break
				when 6 then -- switch 1, default:
						l_pos := -1 + 7 -- switch 1; break
				when 7 then -- switch 1, done: | if done | if done | if done
					if not l_return and then (l_left > 0) then
						if l_left < a_len then
							music_halt_or_loop.do_nothing
							if (music_internal_playing).to_boolean then
								music_mixer (a_udata, a_stream + (a_len - l_left), l_left)
							end
						end
					end
				when 8 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	open_music (a_mixer_1: CE_POINTER [SD_SDL_AUDIOSPEC]): INTEGER_32
		do
			if ((create {P_WAVESTREAM_STATIC}).wavstream_init (a_mixer_1)) = 0 then
				add_music_decoder (ce_string ("WAVE"))
			end
			used_mixer.item.item := (a_mixer_1.item).twin
			add_music_decoder (ce_string ("MP3"))
			music_playing.item := create {CE_CLASS_POINTER [PD_MIX_MUSIC]}.make_shared (default_pointer)
			music_stopped.item := 0
			mix_volume_music (128).do_nothing
			ms_per_step.item := ((((a_mixer_1.item.samples).to_real_32).to_double * 1000.0) / (a_mixer_1.item.freq).to_double).truncated_to_integer
			Result := 0
		end

	mix_string_equals (a_str1, a_str2: CE_POINTER [INTEGER_8]): INTEGER_32
		local
			l_str1: CE_POINTER [INTEGER_8]
			l_str2: CE_POINTER [INTEGER_8]
			l_tmp: INTEGER_32
			l_tmp_1: INTEGER_32
			l_break: BOOLEAN
		do
			l_str1 := a_str1
			l_str2 := a_str2
			from
			until
				(l_break)
			loop
				l_break := False
				if (l_str1[0]).to_boolean then
					if not (l_str2[0]).to_boolean then
						l_break := True
					end
				else
					l_break := True
				end
				if not (l_break) then
					l_tmp := toupper (((l_str1[0]).to_natural_8).to_integer_32)
					if l_tmp /= (toupper (((l_str2[0]).to_natural_8).to_integer_32)) then
						l_break := True
					end
				end
				if not (l_break) then
					l_str1 := l_str1 + 1
					l_str2 := l_str2 + 1
				end
			end
			l_break := False
			if not (l_str1[0]).to_boolean then
				if not (l_str2[0]).to_boolean then
					l_tmp_1 := 1
				else
					l_tmp_1 := 0
				end
			else
				l_tmp_1 := 0
			end
			Result := l_tmp_1
		end

	detect_mp3 (a_magic: CE_POINTER [NATURAL_8]): INTEGER_32
		local
			l_return: BOOLEAN
		do
			if (strncmp (create {CE_INTEGER_8_POINTER}.make_cast (a_magic), ce_string ("ID3"), (3).to_natural_32)) = 0 then
				Result := 1
				l_return := True
			end
			if not l_return then
				if (((a_magic[0]).to_integer_32).bit_and (255)) /= 255 then
					Result := 0
					l_return := True
				elseif (((a_magic[1]).to_integer_32).bit_and (240)) /= 240 then
					Result := 0
					l_return := True
				elseif (((a_magic[2]).to_integer_32).bit_and (240)) = 0 then
					Result := 0
					l_return := True
				elseif (((a_magic[2]).to_integer_32).bit_and (240)) = 240 then
					Result := 0
					l_return := True
				elseif (((a_magic[2]).to_integer_32).bit_and (12)) = 12 then
					Result := 0
					l_return := True
				elseif (((a_magic[1]).to_integer_32).bit_and (6)) = 0 then
					Result := 0
					l_return := True
				end
			end
			if not l_return then
				Result := 1
			end
		end

	detect_music_type (a_rw: CE_POINTER [SD_SDL_RWOPS]): NATURAL_32
		local
			l_magic: CE_ARRAY [NATURAL_8]
			l_moremagic: CE_ARRAY [NATURAL_8]
			l_start: INTEGER_32
			l_return: BOOLEAN
			l_pos: INTEGER_32
		do
			create {CE_NATURAL_8_ARRAY} l_magic.make (<<5>>)
			create {CE_NATURAL_8_ARRAY} l_moremagic.make (<<9>>)

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_start := ((a_rw.item.seek).integer_32_item ([a_rw, 0, 1]))
					if ((a_rw.item.read).integer_32_item ([a_rw, create {CE_VOID_POINTER}.make_cast (l_magic), 1, 4])) /= 4 then
						sdl_set_error ([ce_string ("Couldn%'t read from RWops")])
						Result := ({PC_MIX_MUSICTYPE}.mus_none)
						l_return := True
					elseif ((a_rw.item.read).integer_32_item ([a_rw, create {CE_VOID_POINTER}.make_cast (l_moremagic), 1, 8])) /= 8 then
						sdl_set_error ([ce_string ("Couldn%'t read from RWops")])
						Result := ({PC_MIX_MUSICTYPE}.mus_none)
						l_return := True
					end
					if l_return then
						l_pos := -1 + 5 -- skip if block
					end
				when 1 then -- if:;
						(a_rw.item.seek).call ([a_rw, l_start, 0])
						l_magic[4] := (('%/0/').code).to_natural_8
						l_moremagic[8] := (('%/0/').code).to_natural_8
						if (strcmp (create {CE_INTEGER_8_POINTER}.make_cast (l_magic), ce_string ("RIFF"))) /= 0 then
							l_pos := -1 + 4 -- skip if block
						end
				when 2 then -- if:
						if (strcmp (create {CE_INTEGER_8_POINTER}.make_cast ((l_moremagic + 4)), ce_string ("WAVE"))) = 0 then
							Result := ({PC_MIX_MUSICTYPE}.mus_wav)
							l_return := True
						else
							l_pos := -1 + 4 -- goto _L
						end
				when 3 then -- if block done, skip else block:
						l_pos := -1 + 5
				when 4 then -- else: | _L:
						if (strcmp (create {CE_INTEGER_8_POINTER}.make_cast (l_magic), ce_string ("FORM"))) = 0 then
							Result := ({PC_MIX_MUSICTYPE}.mus_wav)
							l_return := True
						end
				when 5 then -- if done
					if not l_return then
						if (strcmp (create {CE_INTEGER_8_POINTER}.make_cast (l_magic), ce_string ("OggS"))) = 0 then
							Result := ({PC_MIX_MUSICTYPE}.mus_ogg)
							l_return := True
						end
					end
					if not l_return then
						if (strcmp (create {CE_INTEGER_8_POINTER}.make_cast (l_magic), ce_string ("fLaC"))) = 0 then
							Result := ({PC_MIX_MUSICTYPE}.mus_flac)
							l_return := True
						end
					end
					if not l_return then
						if (strcmp (create {CE_INTEGER_8_POINTER}.make_cast (l_magic), ce_string ("MThd"))) = 0 then
							Result := ({PC_MIX_MUSICTYPE}.mus_mid)
							l_return := True
						end
					end
					if not l_return then
						if (detect_mp3 (l_magic)).to_boolean then
							Result := ({PC_MIX_MUSICTYPE}.mus_mp3)
							l_return := True
						end
					end
					if not l_return then
						Result := ({PC_MIX_MUSICTYPE}.mus_mod)
					end
				when 6 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	mix_load_mus (a_file: CE_POINTER [INTEGER_8]): CE_POINTER [PD_MIX_MUSIC]
		local
			l_rw: CE_POINTER [SD_SDL_RWOPS]
			l_music_0: CE_POINTER [PD_MIX_MUSIC]
			l_type: NATURAL_32
			l_ext: CE_POINTER [INTEGER_8]
			l_return: BOOLEAN
		do
			l_ext := (strrchr (a_file, ('.').code))
			l_rw := sdl_rwfrom_file (a_file, ce_string ("rb"))
			if l_rw.is_void then
				sdl_set_error ([ce_string ("Couldn%'t open %'%%s%'"), a_file])
				Result := (create {CE_CLASS_POINTER [PD_MIX_MUSIC]}.make_shared (default_pointer))
				l_return := True
			end
			if not l_return then
				l_type := {PC_MIX_MUSICTYPE}.mus_none
				l_ext := strrchr (a_file, ('.').code)
				if (not l_ext.is_void) then
					l_ext := l_ext + 1
					if (mix_string_equals (l_ext, ce_string ("WAV"))).to_boolean then
						l_type := {PC_MIX_MUSICTYPE}.mus_wav
					elseif (mix_string_equals (l_ext, ce_string ("MID"))).to_boolean then
						l_type := {PC_MIX_MUSICTYPE}.mus_mid
					elseif (mix_string_equals (l_ext, ce_string ("MIDI"))).to_boolean then
						l_type := {PC_MIX_MUSICTYPE}.mus_mid
					elseif (mix_string_equals (l_ext, ce_string ("KAR"))).to_boolean then
						l_type := {PC_MIX_MUSICTYPE}.mus_mid
					elseif (mix_string_equals (l_ext, ce_string ("OGG"))).to_boolean then
						l_type := {PC_MIX_MUSICTYPE}.mus_ogg
					elseif (mix_string_equals (l_ext, ce_string ("FLAC"))).to_boolean then
						l_type := {PC_MIX_MUSICTYPE}.mus_flac
					elseif (mix_string_equals (l_ext, ce_string ("MPG"))).to_boolean then
						l_type := {PC_MIX_MUSICTYPE}.mus_mp3
					elseif (mix_string_equals (l_ext, ce_string ("MPEG"))).to_boolean then
						l_type := {PC_MIX_MUSICTYPE}.mus_mp3
					elseif (mix_string_equals (l_ext, ce_string ("MP3"))).to_boolean then
						l_type := {PC_MIX_MUSICTYPE}.mus_mp3
					elseif (mix_string_equals (l_ext, ce_string ("MAD"))).to_boolean then
						l_type := {PC_MIX_MUSICTYPE}.mus_mp3
					end
				end
				if l_type = {NATURAL_32} 0 then
					l_type := detect_music_type (l_rw)
				end
				sdl_set_error ([ce_string ("")])
				l_music_0 := mix_load_mustype_rw (l_rw, l_type, 1)
				if l_music_0.is_void then
					if ((sdl_get_error)[0]).to_integer_32 = 0 then
						sdl_free_rw (l_rw)
						sdl_set_error ([ce_string ("Couldn%'t open %'%%s%'"), a_file])
					end
				end
				Result := l_music_0
			end
		end

	mix_load_mus_rw (a_rw: CE_POINTER [SD_SDL_RWOPS]): CE_POINTER [PD_MIX_MUSIC]
		do
			Result := (mix_load_mustype_rw (a_rw, {PC_MIX_MUSICTYPE}.mus_none, 0))
		end

	mix_load_mustype_rw (a_rw: CE_POINTER [SD_SDL_RWOPS]; a_type: NATURAL_32; a_freesrc: INTEGER_32): CE_POINTER [PD_MIX_MUSIC]
		local
			l_type: NATURAL_32
			l_music_0: CE_POINTER [PD_MIX_MUSIC]
			l_magic: CE_ARRAY [NATURAL_8]
			l_start: INTEGER_32
			l_info: CE_ARRAY [SD_SMPEG_INFO]
			l_return: BOOLEAN
			l_pos: INTEGER_32
		do
			create {CE_NATURAL_8_ARRAY} l_magic.make (<<5>>)
			create {CE_CLASS_ARRAY [SD_SMPEG_INFO]} l_info.make (<<1>>)

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_type := a_type
					if a_rw.is_void then
						sdl_set_error ([ce_string ("RWops pointer is NULL")])
						Result := (create {CE_CLASS_POINTER [PD_MIX_MUSIC]}.make_shared (default_pointer))
						l_return := True
					end
					if not l_return and then (l_type = {NATURAL_32} 0) then
						l_type := detect_music_type (a_rw)
						if l_type = {NATURAL_32} 0 then
							Result := (create {CE_CLASS_POINTER [PD_MIX_MUSIC]}.make_shared (default_pointer))
							l_return := True
						end
					end
					if not l_return then
						l_music_0 := create {CE_CLASS_POINTER [PD_MIX_MUSIC]}.make_cast ((malloc ((create {PD_MIX_MUSIC}.make).structure_size.to_natural_32)))
						if l_music_0.is_void then
							sdl_set_error ([ce_string ("Out of memory")])
							Result := (create {CE_CLASS_POINTER [PD_MIX_MUSIC]}.make_shared (default_pointer))
							l_return := True
						end
					end
					if l_return then
						l_pos := -1 + 5 -- skip if block
					end
				when 1 then -- if:
						l_music_0.item.error := 0
						inspect l_type -- switch 1 head
						when 2 then l_pos := -1 + 2
						when 6 then l_pos := -1 + 3
						else l_pos := -1 + 4
						end
				when 2 then -- switch 1, case 2u
						l_start := ((a_rw.item.seek).integer_32_item ([a_rw, 0, 1]))
						if ((a_rw.item.read).integer_32_item ([a_rw, create {CE_VOID_POINTER}.make_cast (l_magic), 1, 4])) /= 4 then
							sdl_set_error ([ce_string ("Couldn%'t read from RWops")])
							Result := (create {CE_CLASS_POINTER [PD_MIX_MUSIC]}.make_shared (default_pointer))
							l_return := True
						end
						if not l_return then
							(a_rw.item.seek).call ([a_rw, l_start, 0])
							l_magic[4] := (('%/0/').code).to_natural_8
							l_music_0.item.type := {PC_MIX_MUSICTYPE}.mus_wav
							l_music_0.item.data.wave := (create {P_WAVESTREAM_STATIC}).wavstream_load_song_rw (a_rw, create {CE_INTEGER_8_POINTER}.make_cast (l_magic), a_freesrc)
							if (l_music_0.item.data.wave).is_void then
								l_music_0.item.error := 1
							end
						end
						l_pos := -1 + 5 -- switch 1; break
				when 3 then -- switch 1, case 6u
						if ((create {P_MIXER_STATIC}).mix_init (4)).to_boolean then
							l_music_0.item.type := {PC_MIX_MUSICTYPE}.mus_mp3
							l_music_0.item.data.mp3 := create {CE_VOID_POINTER}.make_cast (((create {P_DYNAMIC_MP3_STATIC}).smpeg.item.smpeg_new_rwops).pointer_item ([a_rw, l_info, 0]))
							if not (l_info.item.has_audio).to_boolean then
								sdl_set_error ([ce_string ("MPEG file does not have any audio stream.")])
								l_music_0.item.error := 1
							else
								((create {P_DYNAMIC_MP3_STATIC}).smpeg.item.smpeg_actual_spec).call ([l_music_0.item.data.mp3, used_mixer.item])
							end
						else
							l_music_0.item.error := 1
						end
						l_pos := -1 + 5 -- switch 1; break
				when 4 then -- switch 1, default:
						sdl_set_error ([ce_string ("Unrecognized music format")])
						l_music_0.item.error := 1
				when 5 then -- switch 1, done: | if done
					if not l_return then
						if (l_music_0.item.error).to_boolean then
							free (create {CE_VOID_POINTER}.make_cast (l_music_0))
							l_music_0 := create {CE_CLASS_POINTER [PD_MIX_MUSIC]}.make_shared (default_pointer)
						end
						Result := l_music_0
					end
				when 6 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	mix_free_music (a_music_0: CE_POINTER [PD_MIX_MUSIC])
		local
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if a_music_0.is_void then
						l_pos := -1 + 6 -- skip if block
					end
				when 1 then -- if:
						sdl_lock_audio
						if a_music_0.is_equal (music_playing.item) then
							from
							until
								a_music_0.item.fading /= {NATURAL_32} 1
							loop
								sdl_unlock_audio
								sdl_delay ((100).to_natural_32)
								sdl_lock_audio
							end
							if a_music_0.is_equal (music_playing.item) then
								music_internal_halt
							end
						end
						sdl_unlock_audio
						inspect a_music_0.item.type -- switch 1 head
						when 2 then l_pos := -1 + 2
						when 6 then l_pos := -1 + 3
						else l_pos := -1 + 4
						end
				when 2 then -- switch 1, case 2u;
						(create {P_WAVESTREAM_STATIC}).wavstream_free_song (a_music_0.item.data.wave)
						l_pos := -1 + 5 -- switch 1; break
				when 3 then -- switch 1, case 6u;
						((create {P_DYNAMIC_MP3_STATIC}).smpeg.item.smpeg_delete).call ([a_music_0.item.data.mp3])
						l_pos := -1 + 5 -- switch 1; break
				when 4 then -- switch 1, default:
						l_pos := -1 + 5 -- switch 1; break
				when 5 then -- switch 1, done:
						free (create {CE_VOID_POINTER}.make_cast (a_music_0))
				when 6 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	mix_get_music_type (a_music_0: CE_POINTER [PD_MIX_MUSIC]): NATURAL_32
		local
			l_type: NATURAL_32
		do
			l_type := {PC_MIX_MUSICTYPE}.mus_none
			if (not a_music_0.is_void) then
				l_type := a_music_0.item.type
			else
				sdl_lock_audio
				if (not music_playing.item.is_void) then
					l_type := music_playing.item.item.type
				end
				sdl_unlock_audio
			end
			Result := l_type
		end

	music_internal_play (a_music_0: CE_POINTER [PD_MIX_MUSIC]; a_position: REAL_64): INTEGER_32
		local
			l_retval: INTEGER_32
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_retval := 0
					if (not music_playing.item.is_void) then
						music_internal_halt
					end
					music_playing.item := a_music_0
					if a_music_0.item.type /= {NATURAL_32} 3 then
						music_internal_initialize_volume
					end
					inspect a_music_0.item.type -- switch 1 head
					when 2 then l_pos := -1 + 1
					when 6 then l_pos := -1 + 2
					else l_pos := -1 + 3
					end
				when 1 then -- switch 1, case 2u;
					(create {P_WAVESTREAM_STATIC}).wavstream_start (a_music_0.item.data.wave)
					l_pos := -1 + 4 -- switch 1; break
				when 2 then -- switch 1, case 6u;
					((create {P_DYNAMIC_MP3_STATIC}).smpeg.item.smpeg_enableaudio).call ([a_music_0.item.data.mp3, 1]);
					((create {P_DYNAMIC_MP3_STATIC}).smpeg.item.smpeg_enablevideo).call ([a_music_0.item.data.mp3, 0]);
					((create {P_DYNAMIC_MP3_STATIC}).smpeg.item.smpeg_play).call ([music_playing.item.item.data.mp3])
					l_pos := -1 + 4 -- switch 1; break
				when 3 then -- switch 1, default:
					sdl_set_error ([ce_string ("Can%'t play unknown music type")])
					l_retval := -1
					l_pos := -1 + 4 -- switch 1; break
				when 4 then -- switch 1, done:
					if l_retval = 0 then
						if a_position > 0.0 then
							if (music_internal_position (a_position)) < 0 then
								sdl_set_error ([ce_string ("Position not implemented for music type")])
								l_retval := -1
							end
						else
							music_internal_position (0.0).do_nothing
						end
					end
					if l_retval < 0 then
						music_playing.item := create {CE_CLASS_POINTER [PD_MIX_MUSIC]}.make_shared (default_pointer)
					end
					Result := l_retval
				when 5 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	mix_fade_in_music_pos (a_music_0: CE_POINTER [PD_MIX_MUSIC]; a_loops, a_ms: INTEGER_32; a_position: REAL_64): INTEGER_32
		local
			l_loops: INTEGER_32
			l_retval: INTEGER_32
			l_return, l_break: BOOLEAN
		do
			l_loops := a_loops
			if ms_per_step.item = 0 then
				sdl_set_error ([ce_string ("Audio device hasn%'t been opened")])
				Result := (-1)
				l_return := True
			end
			if not l_return and then (a_music_0.is_void) then
				sdl_set_error ([ce_string ("music parameter was NULL")])
				Result := (-1)
				l_return := True
			end
			if not l_return then
				if a_ms.to_boolean then
					a_music_0.item.fading := {PC_MIX_FADING}.mix_fading_in
				else
					a_music_0.item.fading := {PC_MIX_FADING}.mix_no_fading
				end
				a_music_0.item.fade_step := 0
				a_music_0.item.fade_steps := a_ms // ms_per_step.item
				sdl_lock_audio
				from
				until
					(l_break)
				loop
					l_break := False
					if (not music_playing.item.is_void) then
						if music_playing.item.item.fading /= {NATURAL_32} 1 then
							l_break := True
						end
					else
						l_break := True
					end
					if not (l_break) then
						sdl_unlock_audio
						sdl_delay ((100).to_natural_32)
						sdl_lock_audio
					end
				end
				l_break := False
				music_active.item := 1
				if l_loops = 1 then
					l_loops := 0
				end
				music_loops.item := l_loops
				l_retval := music_internal_play (a_music_0, a_position)
				sdl_unlock_audio
				Result := l_retval
			end
		end

	mix_fade_in_music (a_music_0: CE_POINTER [PD_MIX_MUSIC]; a_loops, a_ms: INTEGER_32): INTEGER_32
		do
			Result := (mix_fade_in_music_pos (a_music_0, a_loops, a_ms, 0.0))
		end

	mix_play_music (a_music_0: CE_POINTER [PD_MIX_MUSIC]; a_loops: INTEGER_32): INTEGER_32
		do
			Result := (mix_fade_in_music_pos (a_music_0, a_loops, 0, 0.0))
		end

	music_internal_position (a_position: REAL_64): INTEGER_32
		local
			l_retval: INTEGER_32
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_retval := 0
					inspect music_playing.item.item.type -- switch 1 head
					when 6 then l_pos := -1 + 1
					else l_pos := -1 + 2
					end
				when 1 then -- switch 1, case 6u;
					((create {P_DYNAMIC_MP3_STATIC}).smpeg.item.smpeg_rewind).call ([music_playing.item.item.data.mp3]);
					((create {P_DYNAMIC_MP3_STATIC}).smpeg.item.smpeg_play).call ([music_playing.item.item.data.mp3])
					if a_position > 0.0 then
						((create {P_DYNAMIC_MP3_STATIC}).smpeg.item.smpeg_skip).call ([music_playing.item.item.data.mp3, a_position.truncated_to_real])
					end
					l_pos := -1 + 3 -- switch 1; break
				when 2 then -- switch 1, default:
					l_retval := -1
					l_pos := -1 + 3 -- switch 1; break
				when 3 then -- switch 1, done:
					Result := l_retval
				when 4 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	mix_set_music_position (a_position: REAL_64): INTEGER_32
		local
			l_retval: INTEGER_32
		do
			sdl_lock_audio
			if (not music_playing.item.is_void) then
				l_retval := music_internal_position (a_position)
				if l_retval < 0 then
					sdl_set_error ([ce_string ("Position not implemented for music type")])
				end
			else
				sdl_set_error ([ce_string ("Music isn%'t playing")])
				l_retval := -1
			end
			sdl_unlock_audio
			Result := l_retval
		end

	music_internal_initialize_volume
		do
			if music_playing.item.item.fading = {NATURAL_32} 2 then
				music_internal_volume (0)
			else
				music_internal_volume (music_volume.item)
			end
		end

	music_internal_volume (a_volume: INTEGER_32)
		local
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					inspect music_playing.item.item.type -- switch 1 head
					when 2 then l_pos := -1 + 1
					when 6 then l_pos := -1 + 2
					else l_pos := -1 + 3
					end
				when 1 then -- switch 1, case 2u;
					(create {P_WAVESTREAM_STATIC}).wavstream_set_volume (a_volume)
					l_pos := -1 + 4 -- switch 1; break
				when 2 then -- switch 1, case 6u;
					((create {P_DYNAMIC_MP3_STATIC}).smpeg.item.smpeg_setvolume).call ([music_playing.item.item.data.mp3, ((a_volume.to_real / (128).to_real).to_double * 100.0).truncated_to_integer])
					l_pos := -1 + 4 -- switch 1; break
				when 3 then -- switch 1, default:
					l_pos := -1 + 4 -- switch 1; break
				when 4 then -- switch 1, done:
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	mix_volume_music (a_volume: INTEGER_32): INTEGER_32
		local
			l_volume: INTEGER_32
			l_prev_volume: INTEGER_32
			l_return: BOOLEAN
		do
			l_volume := a_volume
			l_prev_volume := music_volume.item
			if l_volume < 0 then
				Result := l_prev_volume
				l_return := True
			end
			if not l_return then
				if l_volume > 128 then
					l_volume := 128
				end
				music_volume.item := l_volume
				sdl_lock_audio
				if (not music_playing.item.is_void) then
					music_internal_volume (music_volume.item)
				end
				sdl_unlock_audio
				Result := l_prev_volume
			end
		end

	music_internal_halt
		local
			l_return: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					inspect music_playing.item.item.type -- switch 1 head
					when 2 then l_pos := -1 + 1
					when 6 then l_pos := -1 + 2
					else l_pos := -1 + 3
					end
				when 1 then -- switch 1, case 2u;
					(create {P_WAVESTREAM_STATIC}).wavstream_stop
					l_pos := -1 + 4 -- switch 1; break
				when 2 then -- switch 1, case 6u;
					((create {P_DYNAMIC_MP3_STATIC}).smpeg.item.smpeg_stop).call ([music_playing.item.item.data.mp3])
					l_pos := -1 + 4 -- switch 1; break
				when 3 then -- switch 1, default:
					l_return := True
				when 4 then -- switch 1, done:
					if not l_return then
						music_playing.item.item.fading := {PC_MIX_FADING}.mix_no_fading
						music_playing.item := create {CE_CLASS_POINTER [PD_MIX_MUSIC]}.make_shared (default_pointer)
					end
				when 5 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	mix_halt_music: INTEGER_32
		do
			sdl_lock_audio
			if (not music_playing.item.is_void) then
				music_internal_halt
			end
			sdl_unlock_audio
			Result := 0
		end

	mix_fade_out_music (a_ms: INTEGER_32): INTEGER_32
		local
			l_retval: INTEGER_32
			l_fade_steps: INTEGER_32
			l_step: INTEGER_32
			l_old_fade_steps: INTEGER_32
			l_return: BOOLEAN
		do
			l_retval := 0
			if ms_per_step.item = 0 then
				sdl_set_error ([ce_string ("Audio device hasn%'t been opened")])
				Result := 0
				l_return := True
			end
			if not l_return and then (a_ms <= 0) then
				mix_halt_music.do_nothing
				Result := 1
				l_return := True
			end
			if not l_return then
				sdl_lock_audio
				if (not music_playing.item.is_void) then
					l_fade_steps := ((a_ms + ms_per_step.item) - 1) // ms_per_step.item
					if music_playing.item.item.fading = {NATURAL_32} 0 then
						music_playing.item.item.fade_step := 0
					else
						l_old_fade_steps := music_playing.item.item.fade_steps
						if music_playing.item.item.fading = {NATURAL_32} 1 then
							l_step := music_playing.item.item.fade_step
						else
							l_step := (l_old_fade_steps - music_playing.item.item.fade_step) + 1
						end
						music_playing.item.item.fade_step := (l_step * l_fade_steps) // l_old_fade_steps
					end
					music_playing.item.item.fading := {PC_MIX_FADING}.mix_fading_out
					music_playing.item.item.fade_steps := l_fade_steps
					l_retval := 1
				end
				sdl_unlock_audio
				Result := l_retval
			end
		end

	mix_fading_music: NATURAL_32
		local
			l_fading: NATURAL_32
		do
			l_fading := {PC_MIX_FADING}.mix_no_fading
			sdl_lock_audio
			if (not music_playing.item.is_void) then
				l_fading := music_playing.item.item.fading
			end
			sdl_unlock_audio
			Result := l_fading
		end

	mix_pause_music
		do
			music_active.item := 0
		end

	mix_resume_music
		do
			music_active.item := 1
		end

	mix_rewind_music
		do
			mix_set_music_position (0.0).do_nothing
		end

	mix_paused_music: INTEGER_32
		do
			Result := (music_active.item = 0).to_integer
		end

	music_internal_playing: INTEGER_32
		local
			l_playing: INTEGER_32
			l_return: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_playing := 1
					if music_playing.item.is_void then
						Result := 0
						l_return := True
					end
					if l_return then
						l_pos := -1 + 6 -- skip if block
					end
				when 1 then -- if:
						inspect music_playing.item.item.type -- switch 1 head
						when 2 then l_pos := -1 + 2
						when 6 then l_pos := -1 + 3
						else l_pos := -1 + 4
						end
				when 2 then -- switch 1, case 2u
						if not ((create {P_WAVESTREAM_STATIC}).wavstream_active).to_boolean then
							l_playing := 0
						end
						l_pos := -1 + 5 -- switch 1; break
				when 3 then -- switch 1, case 6u
						if (((create {P_DYNAMIC_MP3_STATIC}).smpeg.item.smpeg_status).integer_32_item ([music_playing.item.item.data.mp3])) /= 1 then
							l_playing := 0
						end
						l_pos := -1 + 5 -- switch 1; break
				when 4 then -- switch 1, default:
						l_playing := 0
						l_pos := -1 + 5 -- switch 1; break
				when 5 then -- switch 1, done:
						Result := l_playing
				when 6 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	mix_playing_music: INTEGER_32
		local
			l_playing: INTEGER_32
			l_tmp_0: INTEGER_32
		do
			l_playing := 0
			sdl_lock_audio
			if (not music_playing.item.is_void) then
				if music_loops.item.to_boolean then
					l_tmp_0 := 1
				elseif (music_internal_playing).to_boolean then
					l_tmp_0 := 1
				else
					l_tmp_0 := 0
				end
				l_playing := l_tmp_0
			end
			sdl_unlock_audio
			Result := l_playing
		end

	mix_set_music_cmd (a_command: CE_POINTER [INTEGER_8]): INTEGER_32
		local
			l_return: BOOLEAN
		do
			mix_halt_music.do_nothing
			if (not music_cmd.item.is_void) then
				free (create {CE_VOID_POINTER}.make_cast (music_cmd.item))
				music_cmd.item := create {CE_INTEGER_8_POINTER}.make_shared (default_pointer)
			end
			if (not a_command.is_void) then
				music_cmd.item := create {CE_INTEGER_8_POINTER}.make_cast ((malloc ((strlen (a_command)) + {NATURAL_32} 1)))
				if music_cmd.item.is_void then
					Result := (-1)
					l_return := True
				end
				if not l_return then
					strcpy (music_cmd.item, a_command).do_nothing
				end
			end
			if not l_return then
				Result := 0
			end
		end

	mix_set_synchro_value (a_i: INTEGER_32): INTEGER_32
		do
			Result := (-1)
		end

	mix_get_synchro_value: INTEGER_32
		do
			Result := (-1)
		end

	close_music
		do
			mix_halt_music.do_nothing
			free (create {CE_VOID_POINTER}.make_cast (music_decoders.item))
			music_decoders.item := create {CE_POINTER_POINTER [INTEGER_8]}.make_shared (default_pointer)
			num_decoders_0.item := 0
			ms_per_step.item := 0
		end

	mix_set_sound_fonts (a_paths: CE_POINTER [INTEGER_8]): INTEGER_32
		do
			Result := 1
		end

feature {ANY} -- routine addresses

	music_mixer_address: CE_ROUTINE
		once ("PROCESS")
			create Result.make_agent (agent music_mixer)
		end

end
