class
	PD_CHUNK

inherit
	CE_UTILITIES
		redefine
			copy,
			is_equal
		end
	CE_CLASS
		redefine
			make,
			make_shared,
			make_initialized,
			structure_size,
			offset_of,
			copy,
			is_equal
		end

create
	make,
	make_shared,
	make_initialized

feature {NONE}

	make
		do
			internal_c_pointer := internal_c_pointer.memory_alloc (structure_size)
			is_allocated_memory := True
		end

	make_shared (a_c_pointer: POINTER)
		do
			internal_c_pointer := a_c_pointer
		end

	make_initialized (a_data: TUPLE [
				a_magic: NATURAL_32
				a_length: NATURAL_32
				a_data: CE_POINTER [NATURAL_8]
			])
		require else
			a_data /= Void
		do
			make
			Current.magic := a_data.a_magic
			Current.length := a_data.a_length
			Current.data := a_data.a_data
		end

feature {ANY} -- fields

	magic: NATURAL_32 assign set_magic
		do
			Result := c__magic (internal_c_pointer)
		end

	length: NATURAL_32 assign set_length
		do
			Result := c__length (internal_c_pointer)
		end

	data: CE_POINTER [NATURAL_8] assign set_data
		do
			create {CE_NATURAL_8_POINTER} Result.make_shared (c__data (internal_c_pointer))
		end

feature {ANY} -- field setters

	set_magic (a_magic: NATURAL_32)
		do
			c__set_magic (internal_c_pointer, a_magic)
		end

	set_length (a_length: NATURAL_32)
		do
			c__set_length (internal_c_pointer, a_length)
		end

	set_data (a_data: CE_POINTER [NATURAL_8])
		do
			c__set_data (internal_c_pointer, a_data.c_pointer)
		end

feature {ANY}

	structure_size: INTEGER_32
		external
			"C inline use %"sdl_mixer.h%""
		alias
			"return sizeof(struct Chunk);"
		end

	structure_alignment: INTEGER_32
		external
			"C inline use %"sdl_mixer.h%""
		alias
			"return __alignof__(struct Chunk);"
		end

	offset_of (a_field_name: STRING_8): INTEGER_32
		local
			l_c_str: C_STRING
		do
			create l_c_str.make (a_field_name)
			Result := c__offset_of (l_c_str.item)
		end

feature {ANY}

	copy (a_other: like Current)
		do
			make
			internal_c_pointer.memory_copy (a_other.c_pointer, structure_size)
		end

	is_equal (a_other: like Current): BOOLEAN
		do
			Result := internal_c_pointer.memory_compare (a_other.c_pointer, structure_size)
		end

feature {NONE}

	structure_info: CE_STRUCTURE
		do
			-- dummy for CE_CLASS invariant
			create Result.make_struct (<<>>)
			Result.alignment := structure_alignment
		end

	c__magic (a_c_pointer: POINTER): NATURAL_32
		external
			"C inline use %"sdl_mixer.h%""
		alias
			"return ((struct Chunk *)$a_c_pointer)->magic;"
		end

	c__length (a_c_pointer: POINTER): NATURAL_32
		external
			"C inline use %"sdl_mixer.h%""
		alias
			"return ((struct Chunk *)$a_c_pointer)->length;"
		end

	c__data (a_c_pointer: POINTER): POINTER
		external
			"C inline use %"sdl_mixer.h%""
		alias
			"return (EIF_POINTER)((struct Chunk *)$a_c_pointer)->data;"
		end

	c__set_magic (a_c_pointer: POINTER; a_magic: NATURAL_32)
		external
			"C inline use %"sdl_mixer.h%""
		alias
			"((struct Chunk *)$a_c_pointer)->magic = $a_magic;"
		end

	c__set_length (a_c_pointer: POINTER; a_length: NATURAL_32)
		external
			"C inline use %"sdl_mixer.h%""
		alias
			"((struct Chunk *)$a_c_pointer)->length = $a_length;"
		end

	c__set_data (a_c_pointer: POINTER; a_data: POINTER)
		external
			"C inline use %"sdl_mixer.h%""
		alias
			"((struct Chunk *)$a_c_pointer)->data = $a_data;"
		end

	c__offset_of (a_field_name: POINTER): INTEGER_32
		external
			"C inline use %"sdl_mixer.h%", <string.h>, <stddef.h>, <stdio.h>"
		alias
			"[
				if(strcmp($a_field_name, "magic") == 0) return offsetof(struct Chunk, magic);
				if(strcmp($a_field_name, "length") == 0) return offsetof(struct Chunk, length);
				if(strcmp($a_field_name, "data") == 0) return offsetof(struct Chunk, data);
				fprintf(stderr, "struct Chunk.%s not found\n", $a_field_name);
				return -1;
			]"
		end

end
